@@ -3,17 +3,18 @@ from os import getenv
 from pathlib import Path
 
 import connexion
-from flask import Flask, redirect, send_from_directory
+from flask import Flask, redirect, render_template, send_from_directory
 from redis import Redis
 
 from asu import __version__
+import asu.common
 
 
 def create_app(test_config: dict = None) -> Flask:
     """Create the main Flask application
 
     Args:
-        test_config (dict): A dictionry containing a configuration during tests
+        test_config (dict): A dictionary containing a configuration during tests
 
     Returns:
         Flask: The application
@@ -26,11 +27,13 @@ def create_app(test_config: dict = None) -> Flask:
     cnxn = connexion.FlaskApp(__name__)
     app = cnxn.app
     app.config.from_mapping(
+        JSON_PATH=Path.cwd() / "public/json/v1/",
         REDIS_CONN=Redis(host=redis_host, port=redis_port, password=redis_password),
         TESTING=False,
         DEBUG=False,
-        UPSTREAM_URL="https://downloads.cdn.openwrt.org",
+        UPSTREAM_URL="https://downloads.openwrt.org",
         BRANCHES={},
+        ALLOW_DEFAULTS=False,
     )
 
     if not test_config:
@@ -90,11 +93,17 @@ def create_app(test_config: dict = None) -> Flask:
     app.config["OVERVIEW"] = {
         "latest": latest,
         "branches": branches,
-        "server": {"version": __version__, "contact": "mail@aparcar.org"},
+        "server": {
+            "version": __version__,
+            "contact": "mail@aparcar.org",
+            "allow_defaults": app.config["ALLOW_DEFAULTS"],
+        },
     }
 
     # legacy
-    (app.config["JSON_PATH"] / "branches.json").write_text(json.dumps(branches))
+    (app.config["JSON_PATH"] / "branches.json").write_text(
+        json.dumps(list(branches.values()))
+    )
 
     # tdb
     (app.config["JSON_PATH"] / "latest.json").write_text(json.dumps({"latest": latest}))
@@ -106,6 +115,33 @@ def create_app(test_config: dict = None) -> Flask:
         )
     )
 
+    @app.route("/")
+    def overview():
+        return render_template(
+            "overview.html",
+            branches=branches,
+            defaults=app.config["ALLOW_DEFAULTS"],
+            latest=latest,
+            version=__version__,
+        )
+
+    @app.route("/stats")
+    def stats():
+        branch_stats = {}
+        for branch, data in branches.items():
+            branch_stats.setdefault(branch, {})["profiles"] = asu.common.stats_profiles(
+                branch
+            )[0:5]
+        return render_template(
+            "stats.html",
+            versions=asu.common.stats_versions(),
+            branch_stats=branch_stats,
+        )
+
+    for package, source in app.config.get("MAPPING_ABI", {}).items():
+        if not app.config["REDIS_CONN"].hexists("mapping-abi", package):
+            app.config["REDIS_CONN"].hset("mapping-abi", package, source)
+
     cnxn.add_api("openapi.yml", validate_responses=True)
 
     return app
