@@ -12,6 +12,7 @@ from .common import (
     fingerprint_pubkey_usign,
     get_file_hash,
     get_packages_hash,
+    get_str_hash,
     verify_usign,
 )
 
@@ -59,6 +60,10 @@ class ExtractArchiveError(Exception):
     pass
 
 
+class TooManyPackages(Exception):
+    pass
+
+
 def build(req: dict):
     """Build image request and setup ImageBuilders automatically
 
@@ -107,12 +112,21 @@ def build(req: dict):
 
         ib_hash, ib_archive = ib_search.groups()
 
+        if job:
+            job.meta["imagebuilder_status"] = "download_imagebuilder"
+            job.save_meta()
+
         download_file(ib_archive)
 
         if ib_hash != get_file_hash(cache / ib_archive):
             raise BadChecksumError()
 
         (cache / subtarget).mkdir(parents=True, exist_ok=True)
+
+        if job:
+            job.meta["imagebuilder_status"] = "unpack_imagebuilder"
+            job.save_meta()
+
         extract_archive = subprocess.run(
             ["tar", "--strip-components=1", "-xf", ib_archive, "-C", subtarget],
             cwd=cache,
@@ -134,7 +148,7 @@ def build(req: dict):
         repos_path = cache / subtarget / "repositories.conf"
         repos = repos_path.read_text()
 
-        # speed up downloads with HTTP and CDN
+        # speed up downloads with HTTP and (optionally) CDN
         repos = repos.replace("https://downloads.openwrt.org", req["upstream_url"])
         repos = repos.replace("http://downloads.openwrt.org", req["upstream_url"])
         repos = repos.replace("https", "http")
@@ -237,6 +251,10 @@ def build(req: dict):
         remove_packages = (default_packages | profile_packages) - req["packages"]
         req["packages"] = req["packages"] | set(map(lambda p: f"-{p}", remove_packages))
 
+    if job:
+        job.meta["imagebuilder_status"] = "calculate_packages_hash"
+        job.save_meta()
+
     manifest_run = subprocess.run(
         [
             "make",
@@ -255,9 +273,10 @@ def build(req: dict):
             rmtree(cache / subtarget)
             return build(req)
         else:
-            job.meta["stdout"] = manifest_run.stdout
-            job.meta["stderr"] = manifest_run.stderr
-            job.save_meta()
+            if job:
+                job.meta["stdout"] = manifest_run.stdout
+                job.meta["stderr"] = manifest_run.stderr
+                job.save_meta()
             raise PackageSelectionError()
 
     manifest = dict(map(lambda pv: pv.split(" - "), manifest_run.stdout.splitlines()))
@@ -277,19 +296,35 @@ def build(req: dict):
     packages_hash = get_packages_hash(manifest_packages)
     log.debug(f"Packages Hash {packages_hash}")
 
-    bin_dir = Path(req["version"]) / req["target"] / req["profile"] / packages_hash
+    bin_dir = req["request_hash"]
 
     (req["store_path"] / bin_dir).mkdir(parents=True, exist_ok=True)
 
+    build_cmd = [
+        "make",
+        "image",
+        f"PROFILE={req['profile']}",
+        f"PACKAGES={' '.join(req['packages'])}",
+        f"EXTRA_IMAGE_NAME={packages_hash}",
+        f"BIN_DIR={req['store_path'] / bin_dir}",
+    ]
+
+    if job:
+        job.meta["imagebuilder_status"] = "building_image"
+        job.save_meta()
+
+    if req.get("defaults"):
+        defaults_file = (
+            Path(req["store_path"]) / bin_dir / "files/etc/uci-defaults/99-asu-defaults"
+        )
+        defaults_file.parent.mkdir(parents=True)
+        defaults_file.write_text(req["defaults"])
+        build_cmd.append(f"FILES={req['store_path'] / bin_dir / 'files'}")
+
+    log.debug(f"Running {' '.join(build_cmd)}")
+
     image_build = subprocess.run(
-        [
-            "make",
-            "image",
-            f"PROFILE={req['profile']}",
-            f"PACKAGES={' '.join(req['packages'])}",
-            f"EXTRA_IMAGE_NAME={packages_hash}",
-            f"BIN_DIR={req['store_path'] / bin_dir}",
-        ],
+        build_cmd,
         text=True,
         cwd=cache / subtarget,
         capture_output=True,
@@ -299,12 +334,15 @@ def build(req: dict):
     if job:
         job.meta["stdout"] = image_build.stdout
         job.meta["stderr"] = image_build.stderr
-        job.meta["bin_dir"] = str(bin_dir)
+        job.meta["build_cmd"] = build_cmd
         job.save_meta()
 
     if image_build.returncode:
         raise ImageBuildError()
 
+    if "is too big" in image_build.stderr:
+        raise TooManyPackages()
+
     json_file = Path(req["store_path"] / bin_dir / "profiles.json")
 
     if not json_file.is_file():
@@ -318,9 +356,14 @@ def build(req: dict):
     json_content.update({"manifest": manifest})
     json_content.update(json_content["profiles"][req["profile"]])
     json_content["id"] = req["profile"]
+    json_content["bin_dir"] = str(bin_dir)
     json_content.pop("profiles")
 
     if job:
+        job.connection.sadd(
+            f"builds-{version_code}-{req['target']}", req["request_hash"]
+        )
+
         for package in manifest.keys():
             job.connection.zadd(
                 f"stats-packages-{req['branch_data']['name']}",
@@ -340,4 +383,14 @@ def build(req: dict):
             incr=True,
         )
 
+        job.connection.zadd(
+            "stats-versions",
+            {req["version"]: 1},
+            incr=True,
+        )
+
+        job.connection.incr("stats-images")
+        if req.get("defaults"):
+            job.connection.incr("stats-images-custom")
+
     return json_content
