@@ -5,6 +5,36 @@ import struct
 from pathlib import Path
 
 import nacl.signing
+import redis
+import requests
+from flask import current_app
+
+
+def get_redis():
+    return current_app.config["REDIS_CONN"]
+
+
+def is_modified(url: str) -> bool:
+    r = get_redis()
+
+    modified_local = r.hget("last-modified", url)
+    if modified_local:
+        modified_local = modified_local.decode("utf-8")
+
+    modified_remote = requests.head(url).headers.get("last-modified")
+
+    if modified_local:
+        if modified_local == modified_remote:
+            return False
+
+    if modified_remote:
+        r.hset(
+            "last-modified",
+            url,
+            modified_remote,
+        )
+
+    return True
 
 
 def get_str_hash(string: str, length: int = 32) -> str:
@@ -79,7 +109,11 @@ def get_request_hash(req: dict) -> str:
         get_manifest_hash(req.get("packages_versions", {})),
         str(req.get("diff_packages", False)),
     ]
-    return get_str_hash(" ".join(request_array), 12)
+
+    if "defaults" in req:
+        request_array.append(get_str_hash(req.get("defaults", "")))
+
+    return get_str_hash(" ".join(request_array), 32)
 
 
 def get_packages_hash(packages: list) -> str:
@@ -138,3 +172,19 @@ def verify_usign(sig_file: Path, msg_file: Path, pub_key: str) -> bool:
         return True
     except nacl.exceptions.CryptoError:
         return False
+
+
+def stats_versions():
+    return [
+        (int(s), p.decode("utf-8"))
+        for p, s in get_redis().zrevrange(f"stats-versions", 0, -1, withscores=True)
+    ]
+
+
+def stats_profiles(branch):
+    return [
+        (int(s), p.decode("utf-8"))
+        for p, s in get_redis().zrevrange(
+            f"stats-profiles-{branch}", 0, -1, withscores=True
+        )
+    ]
