@@ -1,19 +1,21 @@
 import email
 import json
-from multiprocessing import Pool
+from datetime import datetime
+from shutil import rmtree
 from time import sleep
 
 import click
 import requests
 from flask import Blueprint, current_app
+from rq import Queue
+from rq.exceptions import NoSuchJobError
+from rq.registry import FinishedJobRegistry
+
+from asu.common import get_redis, is_modified
 
 bp = Blueprint("janitor", __name__)
 
 
-def get_redis():
-    return current_app.config["REDIS_CONN"]
-
-
 def parse_packages_file(url, repo):
     r = get_redis()
     req = requests.get(url)
@@ -40,18 +42,16 @@ def parse_packages_file(url, repo):
                 if source_name != package_name:
                     mapping[package_name] = source_name
             else:
-                current_app.warning(f"Something wired about {package}")
+                current_app.logger.warning(f"Something weird about {package}")
             linebuffer = ""
         else:
             linebuffer += line + "\n"
 
     for package, source in mapping.items():
         if not r.hexists("mapping-abi", package):
-            current_app.logger.info(f"Add ABI mapping {package} -> {source}")
+            current_app.logger.info(f"{repo}: Add ABI mapping {package} -> {source}")
             r.hset("mapping-abi", package, source)
 
-    current_app.logger.debug(f"Found {len(packages)} in {repo}")
-
     return packages
 
 
@@ -81,94 +81,87 @@ def get_packages_arch_repo(branch, arch, repo):
 def update_branch(branch):
     r = get_redis()
 
-    targets = branch["targets"].keys()
+    version_path = branch["path"].format(version=branch["versions"][0])
+    targets = list(
+        filter(
+            lambda t: not t.startswith("."),
+            requests.get(
+                current_app.config["UPSTREAM_URL"]
+                + f"/{version_path}/targets?json-targets"
+            ).json(),
+        )
+    )
 
-    if len(targets) == 0:
+    if not targets:
         current_app.logger.warning("No targets found for {branch['name']}")
         return
 
     r.sadd(f"targets-{branch['name']}", *list(targets))
 
     packages_path = branch["path_packages"].format(branch=branch["name"])
-
-    with Pool(20) as pool:
-        pool.starmap(
-            update_arch_packages, map(lambda a: (branch, a), branch["targets"].values())
-        )
+    packages_path = branch["path_packages"].format(branch=branch["name"])
+    output_path = current_app.config["JSON_PATH"] / packages_path
+    output_path.mkdir(exist_ok=True, parents=True)
 
     for version in branch["versions"]:
         current_app.logger.info(f"Update {branch['name']}/{version}")
-        with Pool() as pool:
-            # TODO: ugly
-            version_path = branch["path"].format(version=version)
-            version_path_abs = current_app.config["JSON_PATH"] / version_path
-            output_path = current_app.config["JSON_PATH"] / packages_path
-            version_path_abs.mkdir(exist_ok=True, parents=True)
-            packages_symlink = version_path_abs / "packages"
+        # TODO: ugly
+        version_path = branch["path"].format(version=version)
+        version_path_abs = current_app.config["JSON_PATH"] / version_path
+        output_path = current_app.config["JSON_PATH"] / packages_path
+        version_path_abs.mkdir(exist_ok=True, parents=True)
+        packages_symlink = version_path_abs / "packages"
 
-            if not packages_symlink.exists():
-                packages_symlink.symlink_to(output_path)
+        if not packages_symlink.exists():
+            packages_symlink.symlink_to(output_path)
 
-            pool.starmap(
-                update_target_packages, map(lambda t: (branch, version, t), targets)
-            )
-            pool.starmap(
-                update_target_profiles, map(lambda t: (branch, version, t), targets)
+        for target in targets:
+            update_target_packages(branch, version, target)
+
+        for target in targets:
+            update_target_profiles(branch, version, target)
+
+        overview = {
+            "branch": branch["name"],
+            "release": version,
+            "image_url": current_app.config["UPSTREAM_URL"]
+            + f"/{version_path}/targets/{{target}}",
+            "profiles": [],
+        }
+
+        for profile_file in (version_path_abs / "targets").rglob("**/*.json"):
+            if profile_file.stem in ["index", "manifest", "overview"]:
+                continue
+            profile = json.loads(profile_file.read_text())
+            overview["profiles"].append(
+                {
+                    "id": profile_file.stem,
+                    "target": profile["target"],
+                    "titles": profile["titles"],
+                }
             )
+        (version_path_abs / "overview.json").write_text(
+            json.dumps(overview, sort_keys=True, separators=(",", ":"))
+        )
 
-            overview = {
-                "branch": branch["name"],
-                "release": version,
-                "image_url": current_app.config["UPSTREAM_URL"]
-                + f"/{version_path}/targets/{{target}}",
-                "profiles": [],
-            }
-
-            for profile_file in (version_path_abs / "targets").rglob("**/*.json"):
-                if profile_file.stem in ["index", "manifest", "overview"]:
-                    continue
-                profile = json.loads(profile_file.read_text())
-                overview["profiles"].append(
-                    {
-                        "id": profile_file.stem,
-                        "target": profile["target"],
-                        "titles": profile["titles"],
-                    }
-                )
-            (version_path_abs / "overview.json").write_text(
-                json.dumps(overview, sort_keys=True, separators=(",", ":"))
-            )
+    for arch in r.smembers(f"architectures-{branch['name']}"):
+        update_arch_packages(branch, arch.decode("utf-8"))
 
 
 def update_target_packages(branch: dict, version: str, target: str):
-    current_app.logger.info(f"Updating packages of {branch['name']}")
+    current_app.logger.info(f"{version}/{target}: Update packages")
+
     version_path = branch["path"].format(version=version)
     r = get_redis()
 
-    packages_modified_local = r.get(
-        f"last-modified-packages-{branch['name']}-{version}-{target}"
-    )
-    if packages_modified_local:
-        packages_modified_local = packages_modified_local.decode("utf-8")
-    packages_modified_remote = requests.head(
+    if not is_modified(
         current_app.config["UPSTREAM_URL"]
         + "/"
         + version_path
-        + f"/targets/{target}/packages/Packages.manifest",
-    ).headers.get("last-modified")
-
-    if packages_modified_local:
-        if packages_modified_local == packages_modified_remote:
-            current_app.logger.debug(
-                f"Skip {branch['name']}/{version}/{target} package update"
-            )
-            return
-
-    if packages_modified_remote:
-        r.set(
-            f"last-modified-packages-{branch['name']}-{version}-{target}",
-            packages_modified_remote,
-        )
+        + f"/targets/{target}/packages/Packages.manifest"
+    ):
+        current_app.logger.debug(f"{version}/{target}: Skip package update")
+        return
 
     packages = get_packages_target_base(branch, version, target)
 
@@ -176,6 +169,8 @@ def update_target_packages(branch: dict, version: str, target: str):
         current_app.logger.warning(f"No packages found for {target}")
         return
 
+    current_app.logger.debug(f"{version}/{target}: Found {len(packages)}")
+
     r.sadd(f"packages-{branch['name']}-{version}-{target}", *list(packages.keys()))
 
     output_path = current_app.config["JSON_PATH"] / version_path / "targets" / target
@@ -190,7 +185,7 @@ def update_target_packages(branch: dict, version: str, target: str):
     (output_path / "index.json").write_text(
         json.dumps(
             {
-                "architecture": branch["targets"][target],
+                "architecture": packages["base-files"]["architecture"],
                 "packages": package_index,
             },
             sort_keys=True,
@@ -207,22 +202,11 @@ def update_arch_packages(branch: dict, arch: str):
     r = get_redis()
 
     packages_path = branch["path_packages"].format(branch=branch["name"])
-    packages_modified_local = r.get(f"last-modified-packages-{branch['name']}-{arch}")
-    if packages_modified_local:
-        packages_modified_local = packages_modified_local.decode("utf-8")
-    packages_modified_remote = requests.head(
+    if not is_modified(
         current_app.config["UPSTREAM_URL"] + f"/{packages_path}/{arch}/feeds.conf"
-    ).headers.get("last-modified")
-
-    if packages_modified_local:
-        if packages_modified_local == packages_modified_remote:
-            current_app.logger.debug(f"Skip {branch['name']}/{arch} package update")
-            return
-
-    if packages_modified_remote:
-        r.set(
-            f"last-modified-packages-{branch['name']}-{arch}", packages_modified_remote
-        )
+    ):
+        current_app.logger.debug(f"{branch['name']}/{arch}: Skip package update")
+        return
 
     packages = {}
 
@@ -233,10 +217,14 @@ def update_arch_packages(branch: dict, arch: str):
 
     # update default repositories afterwards so they overwrite redundancies
     for repo in branch["repos"]:
-        packages.update(get_packages_arch_repo(branch, arch, repo))
+        repo_packages = get_packages_arch_repo(branch, arch, repo)
+        current_app.logger.debug(
+            f"{branch['name']}/{arch}/{repo}: Found {len(repo_packages)} packages"
+        )
+        packages.update(repo_packages)
 
     if len(packages) == 0:
-        current_app.logger.warning(f"No packages found for {arch}")
+        current_app.logger.warning(f"{branch['name']}/{arch}: No packages found")
         return
 
     output_path = current_app.config["JSON_PATH"] / packages_path
@@ -264,7 +252,7 @@ def update_target_profiles(branch: dict, version: str, target: str):
         version(str): Version within branch
         target(str): Target within version
     """
-    current_app.logger.info(f"Checking profiles of {branch['name']}/{version}/{target}")
+    current_app.logger.info(f"{version}/{target}: Update profiles")
     r = get_redis()
     version_path = branch["path"].format(version=version)
     req = requests.head(
@@ -274,43 +262,60 @@ def update_target_profiles(branch: dict, version: str, target: str):
 
     if req.status_code != 200:
         current_app.logger.warning(
-            f"Could not download profiles.json for {version}/{target}"
+            f"{version}/{target}: Could not download profiles.json"
         )
         return
 
-    profiles_modified_local = r.get(
-        f"last-modified-profiles-{branch['name']}-{version}-{target}"
-    )
-    if profiles_modified_local:
-        profiles_modified_local = profiles_modified_local.decode("utf-8")
-
-    profiles_modified_remote = req.headers.get("last-modified")
-
-    if profiles_modified_local:
-        if profiles_modified_local == profiles_modified_remote:
-            current_app.logger.debug(f"Skip {branch['name']}/{version} profiles update")
-            return
-
-    if profiles_modified_remote:
-        r.set(
-            f"last-modified-profiles-{branch['name']}-{version}-{target}",
-            profiles_modified_remote,
-        )
-
-    req = requests.get(
+    profiles_url = (
         current_app.config["UPSTREAM_URL"]
         + f"/{version_path}/targets/{target}/profiles.json"
     )
 
-    metadata = req.json()
+    if not is_modified(profiles_url):
+        current_app.logger.debug(f"{version}/{target}: Skip profiles update")
+        return
+
+    metadata = requests.get(profiles_url).json()
     profiles = metadata.pop("profiles", {})
 
-    current_app.logger.info(f"Found {len(profiles)} profiles")
+    r.sadd(f"architectures-{branch['name']}", metadata["arch_packages"])
+    r.hset(f"architecture-{branch['name']}", target, metadata["arch_packages"])
+
+    queue = Queue(connection=r)
+    registry = FinishedJobRegistry(queue=queue)
+    version_code = r.get(f"revision-{version}-{target}")
+    if version_code:
+        version_code = version_code.decode()
+        for request_hash in r.smembers(f"builds-{version_code}-{target}"):
+            current_app.logger.warning(
+                f"{version_code}/{target}: Delete outdated job build"
+            )
+            try:
+                request_hash = request_hash.decode()
+                registry.remove(request_hash, delete_job=True)
+                rmtree(current_app.config["STORE_PATH"] / request_hash)
+
+            except NoSuchJobError:
+                current_app.logger.warning(f"Job was already deleted")
+        r.delete(f"build-{version_code}-{target}")
+
+    r.set(
+        f"revision-{version}-{target}",
+        metadata["version_code"],
+    )
+
+    current_app.logger.info(f"{version}/{target}: Found {len(profiles)} profiles")
 
     for profile, data in profiles.items():
         for supported in data.get("supported_devices", []):
-            r.hset(f"mapping-{branch['name']}-{version}-{target}", supported, profile)
-            current_app.logger.info(f"Add profile mapping {supported} -> {profile}")
+            if not r.hexists(f"mapping-{branch['name']}-{version}-{target}", supported):
+                current_app.logger.info(
+                    f"{version}/{target}: Add profile mapping {supported} -> {profile}"
+                )
+                r.hset(
+                    f"mapping-{branch['name']}-{version}-{target}", supported, profile
+                )
+
         r.sadd(f"profiles-{branch['name']}-{version}-{target}", profile)
         profile_path = (
             current_app.config["JSON_PATH"]
@@ -321,7 +326,18 @@ def update_target_profiles(branch: dict, version: str, target: str):
         ).with_suffix(".json")
         profile_path.parent.mkdir(exist_ok=True, parents=True)
         profile_path.write_text(
-            json.dumps({**data, **metadata}, sort_keys=True, separators=(",", ":"))
+            json.dumps(
+                {
+                    **metadata,
+                    **data,
+                    "id": profile,
+                    "build_at": datetime.utcfromtimestamp(
+                        int(metadata.get("source_date_epoch", 0))
+                    ).strftime("%Y-%m-%dT%H:%M:%S.%fZ"),
+                },
+                sort_keys=True,
+                separators=(",", ":"),
+            )
         )
 
         data["target"] = target
@@ -337,9 +353,12 @@ def update(interval):
     """
     current_app.logger.info("Init ASU janitor")
     while True:
+        if not current_app.config["BRANCHES"]:
+            current_app.logger.error("No BRANCHES defined in config, nothing to do, exiting")
+            return
         for branch in current_app.config["BRANCHES"].values():
             if not branch.get("enabled"):
-                current_app.logger.info(f"Skip disabled version {branch['name']}")
+                current_app.logger.info(f"{branch['name']}: Skip disabled branch")
                 continue
 
             current_app.logger.info(f"Update {branch['name']}")
